@@ -24,32 +24,40 @@
 
 import sys, time
 from PyQt4 import QtCore, QtGui
-from bzrlib.workingtree import WorkingTree
-from bzrlib.revisiontree import RevisionTree
+
 from bzrlib.revision import CURRENT_REVISION
 
 from bzrlib.plugins.qbzr.lib.i18n import gettext
 from bzrlib.plugins.qbzr.lib.util import (
     BTN_CLOSE,
     QBzrWindow,
-    ThrobberWidget,
+    ToolBarThrobberWidget,
     get_apparent_author_name,
     get_set_encoding,
     runs_in_loading_queue,
+    get_icon,
+    FindToolbar,
+    get_monospace_font,
     )
-from bzrlib.plugins.qbzr.lib.revisionmessagebrowser import LogListRevisionMessageBrowser
 from bzrlib.plugins.qbzr.lib.uifactory import ui_current_widget
 from bzrlib.plugins.qbzr.lib.trace import reports_exception
 from bzrlib.plugins.qbzr.lib.logwidget import LogList
-from bzrlib.plugins.qbzr.lib.logmodel import COL_DATE, RevIdRole
 from bzrlib.plugins.qbzr.lib.lazycachedrevloader import (load_revisions,
                                                          cached_revisions)
-from bzrlib.plugins.qbzr.lib.encoding_selector import EncodingSelector
-from bzrlib.plugins.qbzr.lib.syntaxhighlighter import highlight_document
 from bzrlib.plugins.qbzr.lib.texteditannotate import (AnnotateBarBase,
                                                       AnnotateEditerFrameBase)
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), '''
+from bzrlib.workingtree import WorkingTree
+from bzrlib.revisiontree import RevisionTree
+from bzrlib.plugins.qbzr.lib.revisionmessagebrowser import LogListRevisionMessageBrowser
+from bzrlib.plugins.qbzr.lib.encoding_selector import EncodingMenuSelector
+from bzrlib.plugins.qbzr.lib.syntaxhighlighter import highlight_document
 from bzrlib.plugins.qbzr.lib.revtreeview import paint_revno, get_text_color
-
+from bzrlib.plugins.qbzr.lib import logmodel
+from bzrlib.plugins.qbzr.lib.loggraphviz import BranchInfo
+from bzrlib.patiencediff import PatienceSequenceMatcher as SequenceMatcher
+''')
 
 class AnnotateBar(AnnotateBarBase):
     
@@ -189,12 +197,43 @@ class AnnotatedTextEdit(QtGui.QPlainTextEdit):
                 block = block.next()
             del painter
         QtGui.QPlainTextEdit.paintEvent(self, event)
+    
+    def get_positions(self):
+        """Returns the charator positons for the selection start,
+        selection end, center of the viewport, an the number of lines from
+        the top of the viewport to the center of the viewport."""
+        old_cursor = self.textCursor()
+        old_center = self.cursorForPosition(QtCore.QPoint(0, self.height() / 2))
+        lines_to_center = (old_center.block().blockNumber() -
+                           self.verticalScrollBar().value())
+        
+        return (old_cursor.selectionStart(),
+                old_cursor.selectionEnd(),
+                old_center.position()) , lines_to_center
+    
+    def set_positions(self, new_positions, lines_to_center):
+        new_start, new_end, new_center = new_positions
+        if new_center:
+            new_center_cursor = QtGui.QTextCursor(self.document())
+            new_center_cursor.setPosition(new_center)
+            new_scroll = new_center_cursor.block().blockNumber() - lines_to_center
+            self.verticalScrollBar().setValue(new_scroll)
+        
+        new_selection_cursor = QtGui.QTextCursor(self.document())
+        new_selection_cursor.movePosition(QtGui.QTextCursor.Right,
+                                          QtGui.QTextCursor.MoveAnchor,
+                                          new_start)
+        new_selection_cursor.movePosition(QtGui.QTextCursor.Right,
+                                          QtGui.QTextCursor.KeepAnchor,
+                                          new_end - new_start)
+        self.setTextCursor(new_selection_cursor)        
 
 
 class AnnotateWindow(QBzrWindow):
 
-    def __init__(self, branch, tree, path, fileId, encoding=None, parent=None,
-                 ui_mode=True, loader=None, loader_args=None, no_graph=False):
+    def __init__(self, branch, working_tree, annotate_tree, path, fileId,
+                 encoding=None, parent=None, ui_mode=True, no_graph=False,
+                 loader=None, loader_args=None):
         QBzrWindow.__init__(self,
                             [gettext("Annotate"), gettext("Loading...")],
                             parent, ui_mode=ui_mode)
@@ -203,11 +242,14 @@ class AnnotateWindow(QBzrWindow):
         self.windows = []
 
         self.branch = branch
-        self.tree = tree
-        if isinstance(tree, WorkingTree):
-            self.working_tree = tree
-        else:
-            self.working_tree = None
+        self.annotate_tree = annotate_tree
+        self.working_tree = working_tree
+        if (self.working_tree is None and 
+            isinstance(annotate_tree, WorkingTree)):
+            self.working_tree = annotate_tree
+        
+        self.no_graph = no_graph
+        self.old_lines = None
         
         self.fileId = fileId
         self.path = path
@@ -215,7 +257,7 @@ class AnnotateWindow(QBzrWindow):
         self.loader_func = loader
         self.loader_args = loader_args
 
-        self.throbber = ThrobberWidget(self)
+        self.throbber = ToolBarThrobberWidget(self)
         
         self.text_edit_frame = AnnotateEditerFrameBase(self)
         self.text_edit = AnnotatedTextEdit(self)
@@ -223,10 +265,9 @@ class AnnotateWindow(QBzrWindow):
         self.text_edit.setTextInteractionFlags(
             QtCore.Qt.TextSelectableByMouse|
             QtCore.Qt.TextSelectableByKeyboard)
+        self.text_edit.setLineWrapMode(QtGui.QPlainTextEdit.NoWrap)
         
-        self.text_edit.document().setDefaultFont(
-            QtGui.QFont("Courier New,courier", 
-                        self.text_edit.font().pointSize()))
+        self.text_edit.document().setDefaultFont(get_monospace_font())
         
         self.annotate_bar = AnnotateBar(self.text_edit, self, self.get_revno)
         annotate_spliter = QtGui.QSplitter(QtCore.Qt.Horizontal, self)
@@ -242,8 +283,9 @@ class AnnotateWindow(QBzrWindow):
                      QtCore.SIGNAL("cursorPositionChanged()"),
                      self.edit_cursorPositionChanged)        
         
-        self.log_list = AnnotateLogList(self.processEvents, self.throbber, no_graph, self)
-        self.log_list.header().hideSection(COL_DATE)
+        self.log_list = AnnotateLogList(self.processEvents, self.throbber, self)
+        self.log_list.header().hideSection(logmodel.COL_DATE)
+        self.log_list.parent_annotate_window = self
         self.log_branch_loaded = False
         
         self.connect(self.log_list.selectionModel(),
@@ -252,8 +294,8 @@ class AnnotateWindow(QBzrWindow):
         
         self.message = LogListRevisionMessageBrowser(self.log_list, self)
 
-        self.encoding_selector = EncodingSelector(self.encoding,
-            gettext("Encoding:"),
+        self.encoding_selector = EncodingMenuSelector(self.encoding,
+            gettext("Encoding"),
             self._on_encoding_changed)
 
         hsplitter = QtGui.QSplitter(QtCore.Qt.Horizontal)
@@ -270,17 +312,64 @@ class AnnotateWindow(QBzrWindow):
         splitter.setStretchFactor(0, 5)
         splitter.setStretchFactor(1, 2)
 
-        buttonbox = self.create_button_box(BTN_CLOSE)
-
         vbox = QtGui.QVBoxLayout(self.centralwidget)
-        vbox.addWidget(self.throbber)
+        #vbox.addWidget(self.toolbar)
         vbox.addWidget(splitter)
-        hbox = QtGui.QHBoxLayout()
-        hbox.addWidget(self.encoding_selector)
-        hbox.addWidget(buttonbox)
-        vbox.addLayout(hbox)
         self.text_edit.setFocus()
 
+        self.show_find = QtGui.QAction(get_icon("edit-find"), gettext("Find"), self)
+        self.show_find.setShortcuts(QtGui.QKeySequence.Find)
+        self.show_find.setCheckable(True)
+        
+        self.show_goto_line = QtGui.QAction(get_icon("go-jump"), gettext("Goto Line"), self)
+        self.show_goto_line.setShortcuts((QtCore.Qt.CTRL + QtCore.Qt.Key_L,))
+        self.show_goto_line.setCheckable(True)
+        
+        show_view_menu = QtGui.QAction(get_icon("document-properties"), gettext("&View Options"), self)
+        view_menu = QtGui.QMenu(gettext('View Options'), self)
+        show_view_menu.setMenu(view_menu)
+        
+        word_wrap = QtGui.QAction(gettext("Word Wrap"), self)
+        word_wrap.setCheckable(True)
+        self.connect(word_wrap,
+                     QtCore.SIGNAL("toggled (bool)"),
+                     self.word_wrap_toggle)
+        
+        view_menu.addMenu(self.encoding_selector)
+        view_menu.addAction(word_wrap)
+        
+        toolbar = self.addToolBar(gettext("Annotate"))
+        toolbar.setMovable (False)
+        toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
+        
+        #self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
+        toolbar.addAction(self.show_find)
+        toolbar.addAction(self.show_goto_line)
+        toolbar.addAction(show_view_menu)
+        toolbar.widgetForAction(show_view_menu).setPopupMode(QtGui.QToolButton.InstantPopup)
+        
+        spacer = QtGui.QWidget()
+        spacer.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
+        toolbar.addWidget(spacer) 
+        toolbar.addWidget(self.throbber)
+        
+        self.addToolBarBreak()
+        
+        self.find_toolbar = FindToolbar(self, self.text_edit, self.show_find)
+        self.find_toolbar.hide()
+        self.addToolBar(self.find_toolbar)
+        self.connect(self.show_find,
+                     QtCore.SIGNAL("toggled (bool)"),
+                     self.show_find_toggle)
+        
+        self.goto_line_toolbar = GotoLineToolbar(self, self.show_goto_line)
+        self.goto_line_toolbar.hide()
+        self.addToolBar(self.goto_line_toolbar)
+        
+        self.connect(self.show_goto_line,
+                     QtCore.SIGNAL("toggled (bool)"),
+                     self.show_goto_line_toggle )
+
     def show(self):
         QBzrWindow.show(self)
         QtCore.QTimer.singleShot(0, self.initial_load)
@@ -297,7 +386,7 @@ class AnnotateWindow(QBzrWindow):
         try:
             if self.loader_func is not None:
                 (self.branch,
-                 self.tree,
+                 self.annotate_tree,
                  self.working_tree,
                  self.path, self.fileId) = self.loader_func(*self.loader_args)
                 self.loader_func = self.loader_args = None # kill extra refs...
@@ -307,7 +396,7 @@ class AnnotateWindow(QBzrWindow):
             self.branch.lock_read()
             try:
                 self.set_annotate_title()
-                self.annotate(self.tree, self.fileId, self.path)
+                self.annotate(self.annotate_tree, self.fileId, self.path)
             finally:
                 self.branch.unlock()
         finally:
@@ -315,8 +404,8 @@ class AnnotateWindow(QBzrWindow):
     
     def set_annotate_title(self):
         # and update the title to show we are done.
-        if isinstance(self.tree, RevisionTree):
-            revno = self.get_revno(self.tree.get_revision_id())
+        if isinstance(self.annotate_tree, RevisionTree):
+            revno = self.get_revno(self.annotate_tree.get_revision_id())
             if revno:
                 self.set_title_and_icon([gettext("Annotate"), self.path,
                                          gettext("Revision %s") % revno])
@@ -325,19 +414,26 @@ class AnnotateWindow(QBzrWindow):
         self.set_title_and_icon([gettext("Annotate"), self.path])
 
     def get_revno(self, revid):
-        if revid in self.log_list.graph_provider.revid_rev:
-            return self.log_list.graph_provider.revid_rev[revid].revno_str
+        gv = self.log_list.log_model.graph_viz
+        if (gv and
+            revid in gv.revid_rev):
+            return gv.revid_rev[revid].revno_str
         return ""
     
-    def annotate(self, tree, fileId, path):
+    def annotate(self, annotate_tree, fileId, path):
         self.now = time.time()
         self.rev_indexes = {}
         last_revid = None
         lines = []
         annotate = []
         ordered_revids = []
+        
+        
         self.processEvents()
-        for revid, text in tree.annotate_iter(fileId):
+        for revid, text in annotate_tree.annotate_iter(fileId):
+            if revid == CURRENT_REVISION:
+                revid = CURRENT_REVISION + annotate_tree.basedir
+            
             text = text.decode(self.encoding, 'replace')
             
             lines.append(text)
@@ -355,8 +451,20 @@ class AnnotateWindow(QBzrWindow):
             if len(annotate) % 100 == 0:
                 self.processEvents()
         annotate.append((None, False))  # because the view has one more line
-
+        
+        new_positions = None
+        if self.old_lines:
+            # Try keep the scroll, and selection stable.
+            old_positions, lines_to_center = self.text_edit.get_positions()
+            new_positions = self.translate_positions(
+                                    self.old_lines, lines, old_positions)
+            
+        self.text_edit.annotate = None
         self.text_edit.setPlainText("".join(lines))
+        if new_positions:
+            self.text_edit.set_positions(new_positions, lines_to_center)
+        
+        self.old_lines = lines
         self.annotate_bar.adjustWidth(len(lines), 999)
         self.annotate_bar.annotate = annotate
         self.text_edit.annotate = annotate
@@ -367,32 +475,25 @@ class AnnotateWindow(QBzrWindow):
         just_loaded_log = False
         if not self.log_branch_loaded:
             self.log_branch_loaded = True
-            self.log_list.load_branch(self.branch, [self.fileId], tree)
+            bi = BranchInfo('', self.working_tree, self.branch)
+            self.log_list.load(
+                (bi,), bi, [self.fileId], self.no_graph,
+                logmodel.WithWorkingTreeGraphVizLoader)
             
-            self.log_list.context_menu.addAction(
-                                    gettext("&Annotate this revision"),
-                                    self.set_annotate_revision)
+            gv = self.log_list.log_model.graph_viz
+            self.annotate_bar.adjustWidth(len(lines),
+                                          gv.revisions[0].revno_sequence[0])
             
             just_loaded_log = True
             
-            # Show the revisions the we know about now.
-            gp = self.log_list.graph_provider
-            gp.filter_file_id = [False for i in xrange(len(gp.revisions))]
-            
-            changed_indexes = []
+            # Show the revisions the we know about from the annotate.
+            filter = self.log_list.log_model.file_id_filter
+            changed_revs = []
             for revid in self.rev_indexes.keys():
-                index = gp.revid_rev[revid].index
-                gp.filter_file_id[index] = True
-                changed_indexes.append(index)
-                
-                if len(changed_indexes) >=500:
-                    gp.invaladate_filter_cache_revs(changed_indexes)
-                    changed_indexes = []
-            
-            gp.invaladate_filter_cache_revs(changed_indexes, last_call=True)
-            
-            self.annotate_bar.adjustWidth(len(lines),
-                                          gp.revisions[0].revno_sequence[0])
+                rev = gv.revid_rev[revid]
+                filter.filter_file_id[rev.index] = True
+                changed_revs.append(rev)
+            filter.filter_changed_callback(changed_revs, last_call=True)
         
         self.processEvents()
         highlight_document(self.text_edit, path)
@@ -404,16 +505,43 @@ class AnnotateWindow(QBzrWindow):
         
         if just_loaded_log:
             # Check for any other revisions we don't know about
-            revids = [rev.revid for rev in gp.revisions
-                      if rev.revid not in self.rev_indexes]
             
-            for repo, revids in gp.get_repo_revids(revids):
-                chunk_size = 500
-                
-                for start in xrange(0, len(revids), chunk_size):
-                    gp.load_filter_file_id_chunk(repo, 
-                            revids[start:start + chunk_size])
-            gp.load_filter_file_id_chunk_finished()
+            filter = self.log_list.log_model.file_id_filter
+            revids = [rev.revid for rev in gv.revisions
+                      if rev.revid not in self.rev_indexes]
+            filter.load(revids)
+    
+    def translate_positions(self, old_lines, new_lines, old_positions):
+        sm = SequenceMatcher(None, old_lines, new_lines)
+        opcodes = sm.get_opcodes()
+        new_positions = [None for x in range(len(old_positions))]
+        opcode_len = lambda start, end, lines: sum(
+            [len(l) for l in lines[start:end]])
+        for i, old_pos in enumerate(old_positions):
+            old_char_start = 0
+            new_char_start = 0
+            for code, old_start, old_end, new_start, new_end in opcodes:
+                old_len = opcode_len(old_start, old_end, old_lines)
+                new_len = opcode_len(new_start, new_end, new_lines)
+                if (old_pos >= old_char_start and
+                    old_pos < old_char_start + old_len):
+                    if code == 'delete':
+                        new_pos = new_char_start
+                    elif (code == 'replace' and len(opcodes)>1):
+                        # XXX This should cache the opcodes if we do the same
+                        # block more than once.
+                        new_inner_pos = self.translate_positions(
+                            ''.join(old_lines[old_start:old_end]),
+                            ''.join(new_lines[new_start:new_end]),
+                            [old_pos - old_char_start])[0]
+                        new_pos = new_char_start + new_inner_pos
+                    else:
+                        new_pos = new_char_start + (old_pos - old_char_start)
+                    new_positions[i] = new_pos
+                    break
+                old_char_start += old_len
+                new_char_start += new_len
+        return new_positions
     
     def revisions_loaded(self, revisions, last_call):
         for rev in revisions.itervalues():
@@ -442,11 +570,7 @@ class AnnotateWindow(QBzrWindow):
             self.text_edit.textCursor().position()).blockNumber()
         if self.text_edit.annotate:
             rev_id, is_top = self.text_edit.annotate[current_line]
-            if self.log_list.graph_provider.has_rev_id(rev_id):
-                self.log_list.log_model.ensure_rev_visible(rev_id)
-                index = self.log_list.log_model.indexFromRevId(rev_id)
-                index = self.log_list.filter_proxy_model.mapFromSource(index)
-                self.log_list.setCurrentIndex(index)
+            self.log_list.select_revid(rev_id)
 
     @runs_in_loading_queue
     def set_annotate_revision(self):
@@ -454,15 +578,16 @@ class AnnotateWindow(QBzrWindow):
         try:
             self.branch.lock_read()
             try:
-                revid = str(self.log_list.currentIndex().data(RevIdRole).toString())
-                if revid == CURRENT_REVISION:
-                    self.tree = self.working_tree
+                revid = str(self.log_list.currentIndex().data(logmodel.RevIdRole).toString())
+                if revid.startswith(CURRENT_REVISION):
+                    rev = cached_revisions[revid]
+                    self.annotate_tree = self.working_tree
                 else:
-                    self.tree = self.branch.repository.revision_tree(revid)
-                self.path = self.tree.id2path(self.fileId)
+                    self.annotate_tree = self.branch.repository.revision_tree(revid)
+                self.path = self.annotate_tree.id2path(self.fileId)
                 self.set_annotate_title()
                 self.processEvents()
-                self.annotate(self.tree, self.fileId, self.path)
+                self.annotate(self.annotate_tree, self.fileId, self.path)
             finally:
                 self.branch.unlock()
         finally:
@@ -476,7 +601,7 @@ class AnnotateWindow(QBzrWindow):
         try:
             self.branch.lock_read()
             try:
-                self.annotate(self.tree, self.fileId, self.path)
+                self.annotate(self.annotate_tree, self.fileId, self.path)
             finally:
                 self.branch.unlock()
         finally:
@@ -486,16 +611,113 @@ class AnnotateWindow(QBzrWindow):
         revids = self.log_list.get_selection_and_merged_revids()
         self.annotate_bar.highlight_revids = revids
         self.annotate_bar.update()
+    
+    def show_find_toggle(self, state):
+        if state:
+            self.show_goto_line.setChecked(False)
+
+    def show_goto_line_toggle(self, state):
+        self.goto_line_toolbar.setVisible(state)
+        if state:
+            self.goto_line_toolbar.line_edit.setFocus()
+            self.show_find.setChecked(False)
+        else:
+            self.goto_line_toolbar.line_edit.setText('')
+    
+    def word_wrap_toggle(self, state):
+        if state:
+            self.text_edit.setLineWrapMode(QtGui.QPlainTextEdit.WidgetWidth)
+        else:
+            self.text_edit.setLineWrapMode(QtGui.QPlainTextEdit.NoWrap)
+
+# QIntValidator did not work on vila's setup, so this is a workaround.
+class IntValidator(QtGui.QValidator):
+    def validate (self, input, pos):
+        if input == '':
+            return (QtGui.QValidator.Intermediate, pos)
+        try: 
+            i = int(input)
+        except ValueError:
+            return (QtGui.QValidator.Invalid, pos)
+        if i > 0:
+            return (QtGui.QValidator.Acceptable, pos)
+        else:
+            return (QtGui.QValidator.Invalid, pos)
+
+class GotoLineToolbar(QtGui.QToolBar):
+    
+    def __init__(self, anotate_window, show_action):
+        QtGui.QToolBar.__init__(self, gettext("Goto Line"), anotate_window)
+        self.anotate_window = anotate_window
+        if 0: self.anotate_window = AnnotateWindow()
+        self.show_action = show_action
+        
+        self.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
+        self.setMovable (False)
+        
+        label = QtGui.QLabel(gettext("Goto Line: "), self)
+        self.addWidget(label)
+        
+        self.line_edit = QtGui.QLineEdit(self)
+        self.line_edit.setValidator(IntValidator(self.line_edit))
+        self.addWidget(self.line_edit)
+        label.setBuddy(self.line_edit)
+        
+        go = self.addAction(get_icon("go-next"), gettext("Go"))
+        
+        spacer = QtGui.QWidget()
+        spacer.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
+        self.addWidget(spacer)
+        
+        close = QtGui.QAction(self)
+        close.setIcon(self.style().standardIcon(
+                                        QtGui.QStyle.SP_DialogCloseButton))
+        self.addAction(close)
+        close.setShortcut((QtCore.Qt.Key_Escape))
+        close.setShortcutContext(QtCore.Qt.WidgetWithChildrenShortcut)
+        close.setStatusTip(gettext("Close Goto Line"))
+        self.connect(close,
+                     QtCore.SIGNAL("triggered(bool)"),
+                     self.close_triggered)
+        self.connect(go,
+                     QtCore.SIGNAL("triggered(bool)"),
+                     self.go_triggered)
+        self.connect(self.line_edit,
+                     QtCore.SIGNAL("returnPressed()"),
+                     self.go_triggered)
+    
+    def close_triggered(self, state):
+        self.show_action.setChecked(False)
+    
+    def go_triggered(self, state=True):
+        try:
+            line = int(str(self.line_edit.text()))-1
+        except ValueError:
+            pass
+        else:
+            doc = self.anotate_window.text_edit.document()
+            cursor = QtGui.QTextCursor(doc)
+            cursor.setPosition(doc.findBlockByNumber(line).position())
+            self.anotate_window.text_edit.setTextCursor(cursor)
+            
+            self.show_action.setChecked(False)
+
 
 
 class AnnotateLogList(LogList):
-
-    def load(self):
-        self.graph_provider.lock_read_branches()
-        try:
-            self.graph_provider.load_tags()
-            self.log_model.load_graph(
-                self.graph_provider.load_graph_all_revisions_for_annotate)
-            self._adjust_revno_column()
-        finally:
-            self.graph_provider.unlock_branches()
+    
+    parent_annotate_window = None
+    
+    def create_context_menu(self):
+        LogList.create_context_menu(self, diff_is_default_action=False)
+        set_rev_action = QtGui.QAction(gettext("&Annotate this revision"),
+                                       self.context_menu)
+        self.connect(set_rev_action, QtCore.SIGNAL('triggered()'),
+                     self.parent_annotate_window.set_annotate_revision)
+        self.context_menu.insertAction(
+            self.context_menu.actions()[0],
+            set_rev_action)
+        self.context_menu.setDefaultAction(set_rev_action)
+    
+    def default_action(self, index=None):
+        self.parent_annotate_window.set_annotate_revision()
