@@ -18,6 +18,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+import signal
+
 from bzrlib import errors
 from bzrlib.commands import Command
 from bzrlib.option import Option
@@ -157,6 +159,9 @@ class QBzrCommand(Command):
         try:
             ui.ui_factory = QUIFactory()
             
+            # Handle interupt signal correctly.
+            signal.signal(signal.SIGINT, signal.SIG_DFL)
+            
             # Set up global exception handling.
             from bzrlib.plugins.qbzr.lib.trace import excepthook
             sys.excepthook = excepthook
@@ -182,7 +187,11 @@ class QBzrCommand(Command):
             ui.ui_factory = std_ui_factory
 
 
-ui_mode_option = Option("ui-mode", help="Causes dialogs to wait after the operation is complete.")
+ui_mode_option = Option("ui-mode",
+    help="Causes dialogs to wait after the operation is complete.")
+execute_option = Option("execute", short_name='e',
+    help="Causes dialogs to start the underlying action immediately without "
+         "waiting for user input.")
 
 # A special option so 'revision' can be passed as a simple string, when we do
 # *not* wan't bzrlib's feature of parsing the revision string before passing it.
@@ -271,7 +280,7 @@ class cmd_qannotate(QBzrCommand):
 
     def _qbzr_run(self, filename=None, revision=None, encoding=None,
                   ui_mode=False, no_graph=False):
-        win = AnnotateWindow(None, None, None, None,
+        win = AnnotateWindow(None, None, None, None, None,
                              encoding=encoding, ui_mode=ui_mode,
                              loader=self._load_branch,
                              loader_args=(filename, revision),
@@ -288,7 +297,7 @@ class cmd_qadd(QBzrCommand):
     def _qbzr_run(self, selected_list=None, ui_mode=False):
         tree, selected_list = builtins.tree_files(selected_list)
         if selected_list == ['']:
-            selected_list = []
+            selected_list = None
         self.main_window = AddWindow(tree, selected_list, dialog=False, ui_mode=ui_mode)
         self.main_window.show()
         self._application.exec_()
@@ -302,7 +311,7 @@ class cmd_qrevert(QBzrCommand):
     def _qbzr_run(self, selected_list=None, ui_mode=False, no_backup=False):
         tree, selected_list = builtins.tree_files(selected_list)
         if selected_list == ['']:
-            selected_list = []
+            selected_list = None
         self.main_window = RevertWindow(tree, selected_list, dialog=False,
             ui_mode=ui_mode,
             backup=not no_backup)
@@ -344,14 +353,26 @@ class cmd_qcommit(QBzrCommand):
     takes_options = [
             bzr_option('commit', 'message'),
             bzr_option('commit', 'local'),
+            bzr_option('commit', 'file'),
+            Option('file-encoding', type=check_encoding,
+               help='Encoding of commit message file content.'),
             ui_mode_option,
             ]
     aliases = ['qci']
 
-    def _qbzr_run(self, selected_list=None, message=None, local=False, ui_mode=False):
+    def _qbzr_run(self, selected_list=None, message=None, file=None,
+                  local=False, ui_mode=False, file_encoding=None):
+        if message is not None and file:
+            raise errors.BzrCommandError("please specify either --message or --file")
+        if file:
+            f = open(file)
+            try:
+                message = f.read().decode(file_encoding or osutils.get_user_encoding())
+            finally:
+                f.close()
         tree, selected_list = builtins.tree_files(selected_list)
         if selected_list == ['']:
-            selected_list = []
+            selected_list = None
         self.main_window = CommitWindow(tree, selected_list, dialog=False,
             message=message, local=local, ui_mode=ui_mode)
         self.main_window.show()
@@ -378,48 +399,23 @@ class cmd_qdiff(QBzrCommand, DiffArgProvider):
         takes_options.append('change')
     aliases = ['qdi']
 
-    def get_diff_window_args(self, processEvents):
+    def get_diff_window_args(self, processEvents, add_cleanup):
         try:
-            from bzrlib.diff import get_trees_and_branches_to_diff
-            has_get_trees_and_branches_to_diff = True
+            from bzrlib.diff import get_trees_and_branches_to_diff_locked
         except ImportError:
-            from bzrlib.diff import _get_trees_to_diff
-            has_get_trees_and_branches_to_diff = False
-        
-        if has_get_trees_and_branches_to_diff:
+            from bzrlib.diff import get_trees_and_branches_to_diff
             (old_tree, new_tree,
              old_branch, new_branch,
              specific_files, extra_trees) = \
-               get_trees_and_branches_to_diff(self.file_list, self.revision,
-                                              self.old, self.new)
+                get_trees_and_branches_to_diff(
+                    self.file_list, self.revision, self.old, self.new)
         else:
-            old_tree, new_tree, specific_files, extra_trees = \
-                    _get_trees_to_diff(self.file_list, self.revision,
-                                       self.old, self.new)
-            processEvents()
-            
-            if self.file_list:
-                default_location = self.file_list[0]
-            else:
-                # If no path is given, the current working tree is used
-                default_location = CUR_DIR
-
-            self_old = self.old
-            if self_old is None:
-                self_old = default_location
-            wt, old_branch, rp = \
-                BzrDir.open_containing_tree_or_branch(self_old)
-            processEvents()
-            self_new = self.new
-            if self_new is None:
-                self_new = default_location
-            if self_new != self_old :
-                wt, new_branch, rp = \
-                    BzrDir.open_containing_tree_or_branch(self_new)
-            else:
-                new_branch = old_branch
-            processEvents()
-            
+            (old_tree, new_tree,
+             old_branch, new_branch,
+             specific_files, extra_trees) = \
+                get_trees_and_branches_to_diff_locked(
+                    self.file_list, self.revision, self.old, self.new,
+                    add_cleanup)
         return old_tree, new_tree, old_branch, new_branch, specific_files
     
     def get_ext_diff_args(self, processEvents):
@@ -491,13 +487,17 @@ class cmd_qlog(QBzrCommand):
     """
 
     takes_args = ['locations*']
-    takes_options = [ui_mode_option,
-                   	 Option('no-graph', help="Shows the log with no graph."),
-                    ]
+    takes_options = [
+        ui_mode_option,
+        Option('no-graph', help="Shows the log with no graph."),
+        Option('show-trees', help="Show working trees that have changes "
+                                  "as nodes in the graph"),
+        ]
 
-    def _qbzr_run(self, locations_list=None, ui_mode=False, no_graph=False):
+    def _qbzr_run(self, locations_list=None, ui_mode=False, no_graph=False,
+                  show_trees=False):
         window = LogWindow(locations_list, None, None, ui_mode=ui_mode,
-                           no_graph=no_graph)
+                           no_graph=no_graph, show_trees=show_trees)
         window.show()
         self._application.exec_()
 
@@ -710,7 +710,7 @@ class cmd_merge(bzrlib.builtins.cmd_merge, DiffArgProvider):
             del kw['encoding']
         return bzrlib.builtins.cmd_merge.run(self, *args, **kw)
 
-    def get_diff_window_args(self, processEvents):
+    def get_diff_window_args(self, processEvents, add_cleanup):
         tree_merger = self.merger.make_merger()
         self.tt = tree_merger.make_preview_transform()
         result_tree = self.tt.get_preview_tree()
@@ -792,16 +792,16 @@ class cmd_qgetupdates(QBzrCommand):
     """Fetches external changes into the working tree."""
 
     takes_args = ['location?']
-    takes_options = [ui_mode_option]
+    takes_options = [ui_mode_option, execute_option]
     aliases = ['qgetu', 'qgetup']
 
-    def _qbzr_run(self, location=CUR_DIR, ui_mode=False):
+    def _qbzr_run(self, location=CUR_DIR, ui_mode=False, execute=False):
         branch, relpath = Branch.open_containing(location)
         tb = TreeBranch.open_containing(location, ui_mode=ui_mode)
         if tb is None:
             return errors.EXIT_ERROR
         if tb.is_light_co():
-            window = QBzrUpdateWindow(tb.tree, ui_mode)
+            window = QBzrUpdateWindow(tb.tree, ui_mode, immediate=execute)
         elif tb.is_bound():
             window = UpdateCheckoutWindow(tb.branch, ui_mode=ui_mode)
         else:
@@ -922,13 +922,13 @@ class cmd_qupdate(QBzrCommand):
     """Update working tree with latest changes in the branch."""
     aliases = ['qup']
     takes_args = ['directory?']
-    takes_options = [ui_mode_option]
+    takes_options = [ui_mode_option, execute_option]
 
-    def _qbzr_run(self, directory=None, ui_mode=False):
+    def _qbzr_run(self, directory=None, ui_mode=False, execute=False):
         tree = open_tree(directory, ui_mode)
         if tree is None:
             return
-        self.main_window = QBzrUpdateWindow(tree, ui_mode)
+        self.main_window = QBzrUpdateWindow(tree, ui_mode, execute)
         self.main_window.show()
         self._application.exec_()
 
@@ -964,16 +964,16 @@ class cmd_qswitch(QBzrCommand):
 
 class cmd_qunbind(QBzrCommand):
     """Convert the current checkout into a regular branch."""
-    takes_options = [ui_mode_option]
+    takes_options = [ui_mode_option, execute_option]
     
-    def _qbzr_run(self, ui_mode=False):
+    def _qbzr_run(self, ui_mode=False, execute=False):
         from bzrlib.plugins.qbzr.lib.unbind import QBzrUnbindDialog
         
         branch = Branch.open_containing(CUR_DIR)[0]
         if branch.get_bound_location() == None:
             raise errors.BzrCommandError("This branch is not bound.")
         
-        self.main_window = QBzrUnbindDialog(branch, ui_mode)
+        self.main_window = QBzrUnbindDialog(branch, ui_mode, execute)
         self.main_window.show()
         self._application.exec_() 
 
@@ -1043,10 +1043,7 @@ class cmd_qrun(QBzrCommand):
             help='Initial category selection.',
             type=unicode,
             ),
-        Option('execute',
-            help="Validate and run the supplied command immediately.",
-            short_name='e'
-            ),
+        execute_option,
         ]
     aliases = ['qcmd']
 
