@@ -26,13 +26,14 @@ from bzrlib.plugins.qbzr.lib.i18n import gettext
 from bzrlib.plugins.qbzr.lib.util import (
     file_extension,
     format_timestamp,
-    split_tokens_at_lines,
-    CachedTTypeFormater,
-    QBzrGlobalConfig,
-    QBzrConfig,
+    get_qbzr_config,
+    get_monospace_font,
     )
 from bzrlib.trace import mutter
-
+from bzrlib.plugins.qbzr.lib.syntaxhighlighter import (
+    CachedTTypeFormater,
+    split_tokens_at_lines,
+    )
 
 have_pygments = True
 try:
@@ -58,13 +59,13 @@ interline_changes_background = QtGui.QColor(180, 210, 250)
 #  kind_bound -- the color of the boundary of the rectangle this kind refers to.
 #  kind_fill  -- the color of the filling of that same rectangle.
 
-config = QBzrConfig()
+config = get_qbzr_config()
 component_dict = {0:'fill', 1:'bound'}
 for key in colors.iterkeys():
     for comp in [0,1]:
         color = None
         try:
-            color = config.getColor(key + '_' + component_dict[comp],
+            color = config.get_color(key + '_' + component_dict[comp],
                                         'QDIFF COLORS')
         except ValueError, msg:
             #error handling.
@@ -75,8 +76,8 @@ for key in colors.iterkeys():
 
 #Get a user-defined replacement text background
 try:
-    new_interline_bg  = config.getColor('interline_changes_background',
-                                        'QDIFF COLORS')
+    new_interline_bg  = config.get_color('interline_changes_background',
+                                         'QDIFF COLORS')
     if None != new_interline_bg:
       interline_changes_background = new_interline_bg
 except ValueError, msg:
@@ -199,8 +200,7 @@ class SidebySideDiffView(QtGui.QSplitter):
         titleFont.setPointSize(titleFont.pointSize() * 140 / 100)
         titleFont.setBold(True)
         
-        self.monospacedFont = QtGui.QFont("Courier New, Courier",
-                                     self.font().pointSize())
+        self.monospacedFont = get_monospace_font()
         metadataFont = QtGui.QFont(self.font())
         metadataFont.setPointSize(titleFont.pointSize() * 70 / 100)
         metadataLabelFont = QtGui.QFont(metadataFont)
@@ -253,8 +253,8 @@ class SidebySideDiffView(QtGui.QSplitter):
         self.image_exts = ['.'+str(i)
             for i in QtGui.QImageReader.supportedImageFormats()]
         
-        config = QBzrGlobalConfig()
-        self.show_intergroup_colors = config.get_user_option("diff_show_intergroup_colors") in ("True", "1")
+        config = get_qbzr_config()
+        self.show_intergroup_colors = config.get_option("diff_show_intergroup_colors") in ("True", "1")
     
     def clear(self):
         
@@ -581,8 +581,7 @@ class SimpleDiffView(QtGui.QTextBrowser):
         format = QtGui.QTextCharFormat()
         format.setAnchorNames(["top"])
         self.cursor.insertText("", format)
-        monospacedFont = QtGui.QFont("Courier New, Courier",
-                                     self.font().pointSize())
+        monospacedFont = get_monospace_font()
         self.monospacedFormat = QtGui.QTextCharFormat()
         self.monospacedFormat.setFont(monospacedFont)
 
