@@ -18,7 +18,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-"""I18N and L10N support for QBzr"""
+"""I18N and L10N support"""
 
 import gettext as _gettext
 import os
@@ -40,6 +40,11 @@ def N_(s):
 
 def install():
     global _translation
+    if not os.environ.get('LANGUAGE'):
+        from bzrlib import config
+        lang = config.GlobalConfig().get_user_option('language')
+        if lang:
+            os.environ['LANGUAGE'] = lang
     if sys.platform == 'win32':
         _check_win32_locale()
     _translation = _gettext.translation('qbzr', localedir=_get_locale_dir(), fallback=True)
@@ -51,8 +56,11 @@ def uninstall():
 
 
 def _get_locale_dir():
-    return os.path.join(os.path.realpath(os.path.dirname(__file__)), '..', 'locale')
-
+    localedir = os.path.join(os.path.realpath(os.path.dirname(__file__)), '..', 'locale')
+    if sys.platform.startswith('linux'):
+        if not os.access(localedir, os.R_OK | os.X_OK):
+            localedir = '/usr/share/locale'
+    return localedir
 
 def _check_win32_locale():
     for i in ('LANGUAGE','LC_ALL','LC_MESSAGES','LANG'):
