@@ -4,6 +4,7 @@
 # Copyright (C) 2006 Lukáš Lalinský <lalinsky@gmail.com>
 # Copyright (C) 2008 Gary van der Merwe <garyvdm@gmail.com>
 # Copyright (C) 2009 Alexander Belchenko
+# Copyright (C) 2010 QBzr Developers
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,24 +20,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import codecs
 import os
-import re
-import shlex
-import signal
 import sys
-import tempfile
-import thread
 
 from PyQt4 import QtCore, QtGui
 
-from bzrlib import (
-    bencode,
-    commands,
-    osutils,
-    ui,
-    )
-from bzrlib.option import Option
 
 from bzrlib.plugins.qbzr.lib import MS_WINDOWS
 from bzrlib.plugins.qbzr.lib.i18n import gettext
@@ -48,12 +36,35 @@ from bzrlib.plugins.qbzr.lib.util import (
     QBzrWindow,
     StandardButton,
     ensure_unicode,
+    InfoWidget,
     )
+
+from bzrlib.ui.text import TextProgressView, TextUIFactory
 from bzrlib.plugins.qbzr.lib.trace import (
    report_exception,
    SUB_LOAD_METHOD)
 
-from bzrlib.ui.text import TextProgressView, TextUIFactory
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), '''
+import codecs
+import re
+import shlex
+import signal
+import tempfile
+import thread
+
+from bzrlib import (
+    bencode,
+    commands,
+    osutils,
+    ui,
+    )
+
+from bzrlib.bzrdir import BzrDir
+
+from bzrlib.plugins.qbzr.lib.commit import CommitWindow
+from bzrlib.plugins.qbzr.lib.revert import RevertWindow
+''')
 
 
 # Subprocess service messages markers
@@ -61,6 +72,7 @@ SUB_PROGRESS = "qbzr:PROGRESS:"
 SUB_GETPASS = "qbzr:GETPASS:"
 SUB_GETUSER = "qbzr:GETUSER:"
 SUB_GETBOOL = "qbzr:GETBOOL:"
+SUB_ERROR = "qbzr:ERROR:"
 
 
 class SubProcessWindowBase(object):
@@ -73,7 +85,8 @@ class SubProcessWindowBase(object):
                           ui_mode=True,
                           dialog=True,
                           parent=None,
-                          hide_progress=False):
+                          hide_progress=False,
+                          immediate=False):
         self.restoreSize(name, default_size)
         self._name = name
         self._default_size = default_size
@@ -92,61 +105,89 @@ class SubProcessWindowBase(object):
             QtCore.SIGNAL("finished()"),
             self.on_finished)
         self.connect(self.process_widget,
-            QtCore.SIGNAL("failed()"),
+            QtCore.SIGNAL("failed(QString)"),
             self.on_failed)
         self.connect(self.process_widget,
             QtCore.SIGNAL("error()"),
             self.on_error)
 
-        closeButton = StandardButton(BTN_CLOSE)
-        okButton = StandardButton(BTN_OK)
-        cancelButton = StandardButton(BTN_CANCEL)
+        self.closeButton = StandardButton(BTN_CLOSE)
+        self.okButton = StandardButton(BTN_OK)
+        self.cancelButton = StandardButton(BTN_CANCEL)
 
         # ok button gets disabled when we start.
         QtCore.QObject.connect(self,
                                QtCore.SIGNAL("subprocessStarted(bool)"),
-                               okButton,
+                               self.okButton,
                                QtCore.SLOT("setDisabled(bool)"))
 
         # ok button gets hidden when we finish.
         QtCore.QObject.connect(self,
                                QtCore.SIGNAL("subprocessFinished(bool)"),
-                               okButton,
+                               self.okButton,
                                QtCore.SLOT("setHidden(bool)"))
 
         # close button gets shown when we finish.
         QtCore.QObject.connect(self,
                                QtCore.SIGNAL("subprocessFinished(bool)"),
-                               closeButton,
+                               self.closeButton,
                                QtCore.SLOT("setShown(bool)"))
 
         # cancel button gets disabled when finished.
         QtCore.QObject.connect(self,
                                QtCore.SIGNAL("subprocessFinished(bool)"),
-                               cancelButton,
+                               self.cancelButton,
                                QtCore.SLOT("setDisabled(bool)"))
 
         # ok button gets enabled when we fail.
         QtCore.QObject.connect(self,
                                QtCore.SIGNAL("subprocessFailed(bool)"),
-                               okButton,
+                               self.okButton,
                                QtCore.SLOT("setDisabled(bool)"))
 
         # Change the ok button to 'retry' if we fail.
         QtCore.QObject.connect(self,
                                QtCore.SIGNAL("subprocessFailed(bool)"),
-                               lambda failed: okButton.setText(gettext('&Retry')))
+                               lambda failed: self.okButton.setText(
+                                              gettext('&Retry')))
 
         self.buttonbox = QtGui.QDialogButtonBox(self)
-        self.buttonbox.addButton(okButton,
+        self.buttonbox.addButton(self.okButton,
             QtGui.QDialogButtonBox.AcceptRole)
-        self.buttonbox.addButton(closeButton,
+        self.buttonbox.addButton(self.closeButton,
             QtGui.QDialogButtonBox.AcceptRole)
-        self.buttonbox.addButton(cancelButton,
+        self.buttonbox.addButton(self.cancelButton,
             QtGui.QDialogButtonBox.RejectRole)
         self.connect(self.buttonbox, QtCore.SIGNAL("accepted()"), self.do_accept)
         self.connect(self.buttonbox, QtCore.SIGNAL("rejected()"), self.do_reject)
-        closeButton.setHidden(True) # but 'close' starts as hidden.
+        self.closeButton.setHidden(True) # but 'close' starts as hidden.
+
+        self.uncommitted_info = InfoWidget(self)
+        uncommitted_info_layout = QtGui.QHBoxLayout(self.uncommitted_info)
+        
+        # XXX this is to big. Resize
+        uncommitted_info_icon = QtGui.QLabel()
+        uncommitted_info_icon.setPixmap(self.style().standardPixmap(
+            QtGui.QStyle.SP_MessageBoxWarning))
+        uncommitted_info_layout.addWidget(uncommitted_info_icon)
+        
+        uncommitted_info_label = QtGui.QLabel(gettext(
+            'Working tree has uncommitted changes.'))
+        uncommitted_info_layout.addWidget(uncommitted_info_label, 2)
+        uncommitted_info_button_layout = QtGui.QVBoxLayout()
+        uncommitted_info_layout.addLayout(uncommitted_info_button_layout)
+        commit_button = QtGui.QPushButton(gettext('Commit'))
+        self.connect(commit_button, QtCore.SIGNAL("clicked(bool)"),
+                     self.open_commit_win)
+        uncommitted_info_button_layout.addWidget(commit_button)
+        
+        revert_button = QtGui.QPushButton(gettext('Revert'))
+        self.connect(revert_button, QtCore.SIGNAL("clicked(bool)"),
+                     self.open_revert_win)
+        uncommitted_info_button_layout.addWidget(revert_button)
+        self.uncommitted_info.hide()
+        if immediate:
+            self.do_accept()
 
     def make_default_status_box(self):
         status_group_box = QtGui.QGroupBox(gettext("Status"))
@@ -161,6 +202,7 @@ class SubProcessWindowBase(object):
         Button box has 2 buttons: OK and Cancel (after successfull command 
         execution there will be Close and Cancel).
         """
+        yield self.uncommitted_info
         yield self.make_default_status_box()
         yield self.buttonbox
 
@@ -198,7 +240,7 @@ class SubProcessWindowBase(object):
         if self._check_args():
             self.process_widget.do_start(self.dir, *self.args)
         else:
-            self.on_failed()
+            self.on_failed('CheckArgsFailed')
 
     def do_reject(self):
         if self.process_widget.is_running():
@@ -218,9 +260,13 @@ class SubProcessWindowBase(object):
         if not self.ui_mode:
             self.close()
 
-    def on_failed(self):
+    def on_failed(self, error):
         self.emit(QtCore.SIGNAL("subprocessFailed(bool)"), False)
         self.emit(QtCore.SIGNAL("disableUi(bool)"), False)
+        
+        if error=='UncommittedChanges':
+            self.action_url = self.process_widget.error_data['display_url']
+            self.uncommitted_info.show()
 
     def on_error(self):
         self.emit(QtCore.SIGNAL("subprocessError(bool)"), False)
@@ -230,6 +276,27 @@ class SubProcessWindowBase(object):
         if self._restore_size:
             self.resize(self._restore_size)
 
+    def open_commit_win(self, b):
+        # XXX refactor so that the tree can be opened by the window
+        tree, branch = BzrDir.open_tree_or_branch(self.action_url)
+        commit_window = CommitWindow(tree, None)
+        self.windows.append(commit_window)
+        commit_window.show()
+        QtCore.QObject.connect(commit_window,
+                               QtCore.SIGNAL("subprocessFinished(bool)"),
+                               self.uncommitted_info,
+                               QtCore.SLOT("setHidden(bool)"))
+    
+    def open_revert_win(self, b):
+        # XXX refactor so that the tree can be opened by the window
+        tree, branch = BzrDir.open_tree_or_branch(self.action_url)
+        revert_window = RevertWindow(tree, None)
+        self.windows.append(revert_window)
+        revert_window.show()
+        QtCore.QObject.connect(revert_window,
+                               QtCore.SIGNAL("subprocessFinished(bool)"),
+                               self.uncommitted_info,
+                               QtCore.SLOT("setHidden(bool)")) 
 
 class SubProcessWindow(SubProcessWindowBase, QBzrWindow):
 
@@ -277,7 +344,8 @@ class SubProcessDialog(SubProcessWindowBase, QBzrDialog):
                  ui_mode=True,
                  dialog=True,
                  parent=None,
-                 hide_progress=False):
+                 hide_progress=False,
+                 immediate=False):
         QBzrDialog.__init__(self, title, parent)
         self.__init_internal__(title,
                                name=name,
@@ -287,7 +355,8 @@ class SubProcessDialog(SubProcessWindowBase, QBzrDialog):
                                ui_mode=ui_mode,
                                dialog=dialog,
                                parent=parent,
-                               hide_progress=hide_progress)
+                               hide_progress=hide_progress,
+                               immediate=immediate)
 
     def closeEvent(self, event):
         if not self.process_widget.is_running():
@@ -312,6 +381,7 @@ class SimpleSubProcessDialog(SubProcessDialog):
                  hide_progress=False,
                  auto_start_show_on_failed=False,
                  parent=None,
+                 immediate=False
                  ):
         super(SimpleSubProcessDialog, self).__init__(
                                title,
@@ -322,7 +392,8 @@ class SimpleSubProcessDialog(SubProcessDialog):
                                ui_mode=ui_mode,
                                dialog=dialog,
                                parent=parent,
-                               hide_progress=hide_progress)
+                               hide_progress=hide_progress,
+                               immediate=immediate)
         self.desc = desc
         # create a layout to hold our one label and the subprocess widgets.
         layout = QtGui.QVBoxLayout(self)
@@ -413,6 +484,8 @@ class SubProcessWidget(QtGui.QWidget):
             self.hide_progress()
 
         self._args_file = None  # temp file to pass arguments to qsubprocess
+        self.error_class = ''
+        self.error_data = {}
 
     def hide_progress(self):
         self.progressMessage.setHidden(True)
@@ -482,20 +555,40 @@ class SubProcessWidget(QtGui.QWidget):
 
         if dir is None:
             dir = self.defaultWorkingDir
-
+            
+        self.error_class = ''
+        self.error_data = {}
+        
         self.process.setWorkingDirectory(dir)
         if getattr(sys, "frozen", None) is not None:
-            bzr_exe = sys.argv[0]
-            if sys.frozen == 'windows_exe':
-                # bzrw.exe
-                exe = os.path.join(os.path.dirname(sys.argv[0]), 'bzr.exe')
-                if os.path.isfile(exe):
-                    bzr_exe = exe
+            bzr_exe = sys.executable
+            if os.path.basename(bzr_exe) != "bzr.exe":
+                # Was run from bzrw.exe or tbzrcommand.
+                bzr_exe = os.path.join(os.path.dirname(sys.executable), "bzr.exe")
+                if not os.path.isfile(bzr_exe):
+                    self.reportProcessError(
+                        None, gettext('Could not locate "bzr.exe".'))
             self.process.start(
                 bzr_exe, ['qsubprocess', '--bencode', args])
         else:
+            # otherwise running as python script.
+            # ensure run from bzr, and not others, e.g. tbzrcommand.py
+            script = sys.argv[0]
+            # make absolute, because we may be running in a different
+            # dir.
+            script = os.path.abspath(script)
+            if os.path.basename(script) != "bzr":
+                import bzrlib
+                # are we running directly from a bzr directory?
+                script = os.path.join(bzrlib.__path__[0], "..", "bzr")
+                if not os.path.isfile(script):
+                    # maybe from an installed bzr?
+                    script = os.path.join(sys.prefix, "scripts", "bzr")
+                if not os.path.isfile(script):
+                    self.reportProcessError(
+                        None, gettext('Could not locate "bzr" script.'))
             self.process.start(
-                sys.executable, [sys.argv[0], 'qsubprocess', '--bencode', args])
+                sys.executable, [script, 'qsubprocess', '--bencode', args])
 
     def _setup_stdout_stderr(self):
         if self.stdout is None:
@@ -579,6 +672,10 @@ class SubProcessWidget(QtGui.QWidget):
                 
                 data = (button == QtGui.QMessageBox.Yes)
                 self.process.write(SUB_GETBOOL + bencode.bencode(data) + "\n")
+            elif line.startswith(SUB_ERROR):
+                data = bencode.bdecode(line[len(SUB_ERROR):])
+                self.error_class = data[0]
+                self.error_data = decode_unicode_escape(data[1])
             else:
                 line = line.decode(self.encoding, 'replace')
                 self.logMessageEx(line, 'plain', self.stdout)
@@ -623,23 +720,24 @@ class SubProcessWidget(QtGui.QWidget):
             terminal_stream.write(message)
             terminal_stream.write('\n')
 
-    def reportProcessError(self, error):
+    def reportProcessError(self, error, message=None):
         self.aborting = False
         self.setProgress(1000000, [gettext("Failed!")])
-        if error == QtCore.QProcess.FailedToStart:
-            message = gettext("Failed to start bzr.")
-        else:
-            message = gettext("Error while running bzr. (error code: %d)" % error)
+        if message is None:
+            if error == QtCore.QProcess.FailedToStart:
+                message = gettext("Failed to start bzr.")
+            else:
+                message = gettext("Error while running bzr. (error code: %d)" % error)
         self.logMessage(message, True)
-        self.emit(QtCore.SIGNAL("failed()"))
+        self.emit(QtCore.SIGNAL("failed(QString)"), self.error_class)
 
     def onFinished(self, exitCode, exitStatus):
         self._delete_args_file()
         if self.aborting:
             self.aborting = False
             self.setProgress(1000000, [gettext("Aborted!")])
-            self.emit(QtCore.SIGNAL("failed()"))
-        elif exitCode == 0:
+            self.emit(QtCore.SIGNAL("failed(QString)"), 'Aborted')
+        elif exitCode < 3:
             if self.commands and not self.aborting:
                 self._start_next()
             else:
@@ -648,7 +746,7 @@ class SubProcessWidget(QtGui.QWidget):
                 self.emit(QtCore.SIGNAL("finished()"))
         else:
             self.setProgress(1000000, [gettext("Failed!")])
-            self.emit(QtCore.SIGNAL("failed()"))
+            self.emit(QtCore.SIGNAL("failed(QString)"), self.error_class)
 
     def _create_args_file(self, text):
         """@param text: text to write into temp file,
@@ -786,7 +884,19 @@ def run_subprocess_command(cmd, bencoded=False):
         argv = [unicode(p, 'utf-8') for p in shlex.split(cmd_utf8)]
     else:
         argv = [unicode(p, 'utf-8') for p in bencode.bdecode(cmd_utf8)]
-    return commands.run_bzr(argv)
+    try:
+        return commands.run_bzr(argv)
+    except Exception, e:
+        d = {}
+        for key, val in e.__dict__.iteritems():
+            if not key.startswith('_'):
+                if not isinstance(val, unicode):
+                    val = unicode(val)
+                d[key] = val
+        print "%s%s" % (SUB_ERROR,
+                        bencode.bencode((e.__class__.__name__,
+                                         encode_unicode_escape(d))))
+        raise
 
 
 def sigabrt_handler(signum, frame):
@@ -851,3 +961,21 @@ def bencode_prompt(arg):
 
 def bdecode_prompt(s):
     return bencode.bdecode(s).decode('unicode-escape')
+
+def encode_unicode_escape(obj):
+    if isinstance(obj, dict):
+        result = {}
+        for k,v in obj.iteritems():
+            result[k] = v.encode('unicode-escape')
+        return result
+    else:
+        raise TypeError('encode_unicode_escape: unsupported type: %r' % type(obj))
+
+def decode_unicode_escape(obj):
+    if isinstance(obj, dict):
+        result = {}
+        for k,v in obj.iteritems():
+            result[k] = v.decode('unicode-escape')
+        return result
+    else:
+        raise TypeError('decode_unicode_escape: unsupported type: %r' % type(obj))
