@@ -18,9 +18,23 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 import os, sys
+from PyQt4 import QtCore, QtGui
+
+from bzrlib.plugins.qbzr.lib.i18n import gettext
+
+from bzrlib.plugins.qbzr.lib.revtreeview import (
+    RevisionTreeView,
+    RevNoItemDelegate,
+    )
+from bzrlib.plugins.qbzr.lib.uifactory import ui_current_widget
+from bzrlib.lazy_import import lazy_import
+from bzrlib.plugins.qbzr.lib.lazycachedrevloader import cached_revisions
+from bzrlib.plugins.qbzr.lib.trace import report_exception, SUB_LOAD_METHOD
+
+lazy_import(globals(), '''
 import posixpath  # to use '/' path sep in path.join().
 from time import (strftime, localtime)
-from PyQt4 import QtCore, QtGui
+
 from bzrlib import errors
 from bzrlib.workingtree import WorkingTree
 from bzrlib.revisiontree import RevisionTree
@@ -29,19 +43,12 @@ from bzrlib.conflicts import TextConflict, resolve
 
 from bzrlib.plugins.qbzr.lib.cat import QBzrCatWindow, QBzrViewWindow
 from bzrlib.plugins.qbzr.lib.annotate import AnnotateWindow
-from bzrlib.plugins.qbzr.lib.i18n import gettext
 from bzrlib.plugins.qbzr.lib.log import LogWindow
-from bzrlib.plugins.qbzr.lib.revtreeview import (
-    RevisionTreeView,
-    RevNoItemDelegate,
-    )
-from bzrlib.plugins.qbzr.lib.lazycachedrevloader import cached_revisions
 from bzrlib.plugins.qbzr.lib.util import (
     get_set_encoding,
     get_summary,
     get_apparent_author_name,
     )
-from bzrlib.plugins.qbzr.lib.uifactory import ui_current_widget
 from bzrlib.plugins.qbzr.lib.subprocess import SimpleSubProcessDialog
 from bzrlib.plugins.qbzr.lib.diff import (
     show_diff,
@@ -49,9 +56,8 @@ from bzrlib.plugins.qbzr.lib.diff import (
     ExtDiffMenu,
     InternalWTDiffArgProvider,
     )
-from bzrlib.plugins.qbzr.lib.trace import report_exception, SUB_LOAD_METHOD
-
 
+''')
 def dict_set_add(dict, key, value):
     if key in dict:
         dict[key].add(value)
@@ -63,7 +69,7 @@ def group_large_dirs(paths):
     # deep paths.
     
     all_paths_expanded = {'':('', 0, set([]))}
-    """Dict of all paths expaned, and thier depth, and a set of decendents
+    """Dict of all paths expaned, and their depth, and a set of decendents
     they contain.
     
     The key is the path
@@ -105,8 +111,10 @@ def group_large_dirs(paths):
         while dir_path:
             dir_path, name = os.path.split(dir_path)
             ans_decendents = all_paths_expanded[dir_path][2]
+            old_len = len(ans_decendents)
             ans_decendents.difference_update(decendents)
-            ans_decendents.add(path)
+            if len(ans_decendents) < old_len:
+                ans_decendents.add(path)
     
     # directories included in the original paths container.
     for path, depth, decendents in paths_deep_first:
@@ -387,6 +395,7 @@ class TreeModel(QtCore.QAbstractItemModel):
         self.branch = branch
         self.revno_map = None
         self.changes_mode = changes_mode
+        self.change_load_filter = change_load_filter
         
         self.emit(QtCore.SIGNAL("layoutAboutToBeChanged()"))
         
@@ -419,8 +428,8 @@ class TreeModel(QtCore.QAbstractItemModel):
                         is_ignored = self.tree.is_ignored(path)
                         change = ChangeDesc(change+(is_ignored,))
                         
-                        if (change_load_filter is not None and
-                            not change_load_filter(change)):
+                        if (self.change_load_filter is not None and
+                            not self.change_load_filter(change)):
                             continue
                         
                         item = InternalItem("", change.kind(), fileid)
@@ -562,6 +571,11 @@ class TreeModel(QtCore.QAbstractItemModel):
                                      (None, kind),
                                      (None, executable),
                                      is_ignored))
+                
+                if (self.change_load_filter is not None and
+                    not self.change_load_filter(change)):
+                    continue
+                
                 yield ModelItemData(path, item=child, change=change)
         
         if (not isinstance(item, InternalItem) and
@@ -585,16 +599,33 @@ class TreeModel(QtCore.QAbstractItemModel):
             for path in self.unver_by_parent[item_data.path]:
                 yield self.inventory_data_by_path[path]
     
+    
+    _many_loaddirs_started = False
+    _many_loaddirs_should_start = False
+    def start_maybe_many_loaddirs(self):
+        self._many_loaddirs_should_start = True
+    
+    def end_maybe_many_loaddirs(self):
+        self._many_loaddirs_should_start = False
+        if self._many_loaddirs_started:
+            self._many_loaddirs_started = False
+            self.tree.unlock()
+    
+    
     def load_dir(self, dir_id):
         if dir_id>=len(self.inventory_data):
             return
         dir_item = self.inventory_data[dir_id]
         if dir_item.children_ids is not None:
-            return # This dir has allready been loaded.
+            return # This dir has already been loaded.
         if not dir_item.item.kind=='directory':
             return
         
-        self.tree.lock_read()
+        if not self._many_loaddirs_started:
+            self.tree.lock_read()
+            if self._many_loaddirs_should_start:
+                self._many_loaddirs_started = True
+        
         try:
             dir_item.children_ids = []
             children = sorted(self.get_children(dir_item),
@@ -608,7 +639,8 @@ class TreeModel(QtCore.QAbstractItemModel):
             finally:
                 self.endInsertRows();
         finally:
-            self.tree.unlock()
+            if not self._many_loaddirs_started:
+                self.tree.unlock()
     
     def process_inventory(self, get_children, initial_checked_paths, load_dirs):
         self.get_children = get_children
@@ -755,22 +787,24 @@ class TreeModel(QtCore.QAbstractItemModel):
 
     def setData(self, index, value, role):
 
-        def set_checked(item_data, checked):
-            old_checked = item_data.checked
-            item_data.checked = checked
-            if not old_checked == checked:
-                index = self.createIndex (item_data.row, self.NAME, item_data.id)
-                self.emit(QtCore.SIGNAL("dataChanged(QModelIndex, QModelIndex)"),
-                          index,index)        
-            
         if index.column() == self.NAME and role == QtCore.Qt.CheckStateRole:
+
+            def set_checked(item_data, checked, emit=True):
+                old_checked = item_data.checked
+                item_data.checked = checked
+                return not old_checked == checked
+            
             value = value.toInt()[0]
             if index.internalId() >= len(self.inventory_data):
                 return False
             
             item_data = self.inventory_data[index.internalId()]
+            first_index = self._index_from_id(item_data.id, self.NAME)
             set_checked(item_data, value)
             
+            # this is an array so that it is a poormans nonlocal
+            # http://www.python.org/dev/peps/pep-3104/
+            last_item_data = [None]
             # Recursively set all children to checked.
             def set_child_checked_recurse(item_data):
                 if (item_data.children_ids is None and
@@ -793,6 +827,8 @@ class TreeModel(QtCore.QAbstractItemModel):
                         (change,
                          lookat_children) = self.is_item_in_select_all(child)
                     
+                    last_item_data[0] = child
+                    
                     if lookat_children:
                         has_children_changed = set_child_checked_recurse(child)
                     else:
@@ -800,10 +836,21 @@ class TreeModel(QtCore.QAbstractItemModel):
 
                     if (change or has_children_changed):
                         have_changed_item = True
-                        set_checked(child, value)
+                        set_checked(child, value, False)
                 return have_changed_item
             
-            set_child_checked_recurse(item_data)
+            self.start_maybe_many_loaddirs()
+            try:
+                set_child_checked_recurse(item_data)
+            finally:
+                self.end_maybe_many_loaddirs()
+            
+            if last_item_data[0]:
+                last_index = self._index_from_id(last_item_data[0].id, self.NAME)
+            else:
+                last_index = first_index
+            self.emit(QtCore.SIGNAL("dataChanged(QModelIndex, QModelIndex)"),
+                      first_index,last_index)
             
             # Walk up the tree, and update every dir
             parent_data = item_data
@@ -835,12 +882,17 @@ class TreeModel(QtCore.QAbstractItemModel):
                         break
                 
                 if has_checked and has_unchecked:
-                    set_checked(parent_data, QtCore.Qt.PartiallyChecked)
+                    checked = QtCore.Qt.PartiallyChecked
                 elif has_checked:
-                    set_checked(parent_data, QtCore.Qt.Checked)
+                    checked = QtCore.Qt.Checked
                 else:
-                    set_checked(parent_data, QtCore.Qt.Unchecked)
-            
+                    checked = QtCore.Qt.Unchecked
+                
+                if set_checked(parent_data, checked):
+                    index = self._index_from_id(parent_data.id, self.NAME)
+                    self.emit(QtCore.SIGNAL("dataChanged(QModelIndex, QModelIndex)"),
+                              index,index)        
+
             return True
         
         if index.column() == self.NAME and role == QtCore.Qt.EditRole:
@@ -1137,12 +1189,16 @@ class TreeFilterProxyModel(QtGui.QSortFilterProxyModel):
     
     def invalidateFilter(self):
         self.filter_cache = {}
-        QtGui.QSortFilterProxyModel.invalidateFilter(self)
+        self.source_model.start_maybe_many_loaddirs()
+        try:
+            QtGui.QSortFilterProxyModel.invalidateFilter(self)
+        finally:
+            self.source_model.end_maybe_many_loaddirs()
     
     def setFilter(self, filter, value):
         self.filters[filter] = value
         # This is slow. It causes TreeModel.index, and TreeModel.data thousands
-        # of times. 
+        # of times.
         self.invalidateFilter()
     
     def setFilters(self, filters):
@@ -1156,8 +1212,8 @@ class TreeFilterProxyModel(QtGui.QSortFilterProxyModel):
         self.invalidateFilter()
 
     def filterAcceptsRow(self, source_row, source_parent):
-        if all(self.filters):
-            return True
+        #if all(self.filters):
+        #    return True
         
         model = self.source_model
         parent_id = source_parent.internalId()
@@ -1174,15 +1230,35 @@ class TreeFilterProxyModel(QtGui.QSortFilterProxyModel):
         if id in self.filter_cache:
             return self.filter_cache[id]
         else:
-            result = self.filter_id(id)
+            result = self.filter_id_recurse(id)
             self.filter_cache[id] = result
             return result
     
-    def filter_id(self, id):
+    def filter_id_recurse(self, id):
+        item_data = self.source_model.inventory_data[id]
+        
+        filter = self.filter_id(id, item_data)
+        if filter is not None:
+            return filter
+        
+        if item_data.item.kind == "directory":
+            if item_data.children_ids is None:
+                self.source_model.load_dir(id)
+            
+            for child_id in item_data.children_ids:
+                if self.filter_id_cached(child_id):
+                    return True
+        return False
+    
+    def filter_id(self, id, item_data):
+        """Determines wether a item should be displayed.
+        Returns :
+            * True: Show the item
+            * False: Donot show the item
+            * None: Show the item if there are any children that are visible.
+        """
+        
         (unchanged, changed, unversioned, ignored) = self.filters
-
-        model = self.source_model
-        item_data = model.inventory_data[id]
         
         if item_data.change is None and unchanged: return True
         
@@ -1191,21 +1267,15 @@ class TreeFilterProxyModel(QtGui.QSortFilterProxyModel):
         if is_versioned and item_data.change is not None and changed:
             return True
         
-        if not is_versioned and unversioned:
-            is_ignored = item_data.change.is_ignored()
-            if not is_ignored: return True
-            if is_ignored and ignored: return True
-            if is_ignored and not ignored: return False
-        
-        if item_data.item.kind == "directory":
-            if item_data.children_ids is None:
-                model.load_dir(id)
-            
-            for child_id in item_data.children_ids:
-                if self.filter_id_cached(child_id):
-                    return True
-        
-        return False
+        if not is_versioned:
+            if unversioned or ignored:
+                is_ignored = item_data.change.is_ignored()
+                if not is_ignored and unversioned: return True
+                if is_ignored: return ignored
+            else:
+                return False
+       
+        return None
     
     def on_revisions_loaded(self, revisions, last_call):
         self.source_model.on_revisions_loaded(revisions, last_call)
@@ -1363,6 +1433,8 @@ class TreeWidget(RevisionTreeView):
         if isinstance(tree, RevisionTree) and branch is None:
             raise AttributeError("A branch must be provided if the tree is a "
                                  "RevisionTree")
+        
+        self.revision_loading_disabled = isinstance(tree, WorkingTree)
         self.branch = branch
         self.changes_mode = changes_mode
         self.want_unversioned = want_unversioned
@@ -1777,7 +1849,7 @@ class TreeWidget(RevisionTreeView):
         fileids = [item.item.file_id for item in items
                    if item.item.file_id is not None]
         
-        window = LogWindow(None, self.branch, fileids)
+        window = LogWindow(branch=self.branch,  specific_file_ids=fileids)
         window.show()
         self.window().windows.append(window)
     
@@ -1790,8 +1862,8 @@ class TreeWidget(RevisionTreeView):
 
         if isinstance(file_id, unicode):
             raise errors.InternalBzrError('file_id should be plain string, not unicode')
-
-        window = AnnotateWindow(self.branch, self.tree, path, file_id)
+        
+        window = AnnotateWindow(self.branch, None, self.tree, path, file_id)
         window.show()
         self.window().windows.append(window)
 
@@ -1817,6 +1889,24 @@ class TreeWidget(RevisionTreeView):
         show_diff(arg_provider, ext_diff=ext_diff,
                   parent_window=self.window())
 
+    def unversioned_parents_paths(self, item, include_item=True):
+        paths = []
+        first = True
+        while item.item.file_id is None:
+            if first:
+                if include_item:
+                    paths.append(item.path)
+                first = False
+            else:
+                paths.append(item.path)
+            
+            if item.parent_id:
+                item = self.tree_model.inventory_data[item.parent_id]
+            else:
+                break
+        paths.reverse()
+        return paths
+
     @ui_current_widget
     def add(self):
         """Add selected file(s)."""
@@ -1824,9 +1914,9 @@ class TreeWidget(RevisionTreeView):
         items = self.get_selection_items()
         
         # Only paths that are not versioned.
-        paths = [item.path
-                 for item in items
-                 if item.item.file_id is None]
+        paths = []
+        for item in items:
+            paths.extend(self.unversioned_parents_paths(item))
         
         if len(paths) == 0:
             return
@@ -1847,7 +1937,7 @@ class TreeWidget(RevisionTreeView):
         # Only paths that have changes.
         paths = [item.path
                  for item in items
-                 if item.change is not None and
+               if item.change is not None and
                     item.item.file_id is not None]
         
         if len(paths) == 0:
@@ -1916,12 +2006,12 @@ class TreeWidget(RevisionTreeView):
         except Exception:
             report_exception(type=SUB_LOAD_METHOD, window=self.window())
         self.refresh()
-    
+
     def mark_move(self):
         items = self.get_selection_items()
         if len(items) <> 2:
             return
-        
+
         if missing_unversioned(items[0], items[1]):
             old = items[0]
             new = items[1]
@@ -1931,6 +2021,8 @@ class TreeWidget(RevisionTreeView):
         else:
             return
         try:
+            # add the new parent
+            self.tree.add(self.unversioned_parents_paths(new, False))
             self.tree.rename_one(old.path, new.path, after=True)
         except Exception:
             report_exception(type=SUB_LOAD_METHOD, window=self.window())
