@@ -23,31 +23,40 @@ import shlex
 import sys
 import itertools
 
+
 from PyQt4 import QtCore, QtGui
 
 from bzrlib.revision import Revision
 from bzrlib.config import (
     GlobalConfig,
-    IniBasedConfig,
     config_dir,
     ensure_config_dir_exists,
+    config_filename,
     )
-from bzrlib import (
-    lazy_regex,
-    osutils,
-    urlutils,
-    )
-from bzrlib.util.configobj import configobj
+from bzrlib import lazy_regex
+
 
 from bzrlib.plugins.qbzr.lib import MS_WINDOWS
-from bzrlib.plugins.qbzr.lib import trace
+
 from bzrlib.plugins.qbzr.lib.i18n import gettext, N_
 
 # pyflakes says this is not needed, but it is.
 import bzrlib.plugins.qbzr.lib.resources
 
 from bzrlib import errors
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), '''
+from bzrlib import (
+    osutils,
+    urlutils,
+)
+from bzrlib.util.configobj import configobj
+from bzrlib.plugins.qbzr.lib import trace
 from bzrlib.workingtree import WorkingTree
+from bzrlib.transport import get_transport
+from bzrlib.lockdir import LockDir
+''')
 
 # standard buttons with translatable labels
 BTN_OK, BTN_CANCEL, BTN_CLOSE, BTN_HELP, BTN_REFRESH = range(5)
@@ -87,22 +96,29 @@ class Config(object):
     def __init__(self, filename):
         self._filename = filename
         self._configobj = None
-
+        dir = osutils.dirname(osutils.safe_unicode(filename))
+        transport = get_transport(dir)
+        self._lock = LockDir(transport, 'lock')
+    
     def _load(self):
         if self._configobj is not None:
             return
         self._configobj = configobj.ConfigObj(self._filename,
                                               encoding='utf-8')
 
-    def setOption(self, name, value, section=None):
+    def set_option(self, name, value, section=None):
         self._load()
         if section is None:
             section = 'DEFAULT'
         if section not in self._configobj:
             self._configobj[section] = {}
-        self._configobj[section][name] = value
+        if value:
+            self._configobj[section][name] = value
+        else:
+            if name in self._configobj[section]:
+                del self._configobj[section][name] 
 
-    def getOption(self, name, section=None):
+    def get_option(self, name, section=None):
         self._load()
         if section is None:
             section = 'DEFAULT'
@@ -111,11 +127,11 @@ class Config(object):
         except KeyError:
             return None
 
-    def setSection(self, name, values):
+    def set_section(self, name, values):
         self._load()
         self._configobj[name] = values
 
-    def getSection(self, name):
+    def get_section(self, name):
         self._load()
         try:
             return self._configobj[name]
@@ -123,11 +139,15 @@ class Config(object):
             return {}
 
     def save(self):
-        self._load()
         ensure_config_dir_exists(os.path.dirname(self._filename))
-        f = open(self._filename, 'wb')
-        self._configobj.write(f)
-        f.close()
+        self._lock.lock_write()
+        try:
+            self._load()
+            f = open(self._filename, 'wb')
+            self._configobj.write(f)
+            f.close()
+        finally:
+            self._lock.unlock()
 
 
 class QBzrConfig(Config):
@@ -135,8 +155,8 @@ class QBzrConfig(Config):
     def __init__(self):
         super(QBzrConfig, self).__init__(config_filename())
 
-    def getBookmarks(self):
-        section = self.getSection('BOOKMARKS')
+    def get_bookmarks(self):
+        section = self.get_section('BOOKMARKS')
         i = 0
         while True:
             try:
@@ -147,19 +167,19 @@ class QBzrConfig(Config):
             i += 1
             yield name, location
 
-    def setBookmarks(self, bookmarks):
+    def set_bookmarks(self, bookmarks):
         section = {}
         for i, (name, location) in enumerate(bookmarks):
             section['bookmark%d' % i] = location
             section['bookmark%d_name' % i] = name
-        self.setSection('BOOKMARKS', section)
+        self.set_section('BOOKMARKS', section)
 
-    def addBookmark(self, name, location):
+    def add_bookmark(self, name, location):
         bookmarks = list(self.getBookmarks())
         bookmarks.append((name, location))
         self.setBookmarks(bookmarks)
         
-    def getColor(self, name, section=None):
+    def get_color(self, name, section=None):
         """
           Get a color entry from the config file.
           Color entries have the syntax:
@@ -187,7 +207,7 @@ class QBzrConfig(Config):
             "Color components for " + name_str +\
             " should be in the range 0..255 only. Given: "+ given +"."
             
-        val = self.getOption(name, section)
+        val = self.get_option(name, section)
         if None == val:
           return None
           
@@ -208,22 +228,37 @@ class QBzrConfig(Config):
         return QtGui.QColor(*color_components)
 
 
-class QBzrGlobalConfig(IniBasedConfig):
+_global_config = None
+def get_global_config():
+    global _global_config
+    
+    if (_global_config is None or
+        _check_global_config_filename_valid(_global_config)):
+        _global_config = GlobalConfig()
+    return _global_config
 
-    def __init__(self):
-        super(QBzrGlobalConfig, self).__init__(config_filename)
+def _check_global_config_filename_valid(config):
+    # before bzr 2.3, there was no file_name attrib, only _get_filename, and
+    # checking that would be meaningless.
+    if hasattr(config, 'file_name'):
+        return not config.file_name == config_filename()
+    else:
+        return False
+    
 
-    def set_user_option(self, option, value):
-        """Save option and its value in the configuration."""
-        conf_dir = os.path.dirname(self._get_filename())
-        ensure_config_dir_exists(conf_dir)
-        if 'DEFAULT' not in self._get_parser():
-            self._get_parser()['DEFAULT'] = {}
-        self._get_parser()['DEFAULT'][option] = value
-        f = open(self._get_filename(), 'wb')
-        self._get_parser().write(f)
-        f.close()
+_qbzr_config = None
+def get_qbzr_config():
+    global _qbzr_config
+    if (_qbzr_config is None or
+        not _qbzr_config._filename == config_filename()):
+        _qbzr_config = QBzrConfig()
+    return _qbzr_config
 
+def get_branch_config(branch):
+    if branch is not None:
+        return branch.get_config()
+    else:
+        return get_global_config()
 
 class _QBzrWindowBase(object):
 
@@ -266,7 +301,7 @@ class _QBzrWindowBase(object):
                 QtCore.SIGNAL(signal_name), getattr(self, method_name))
         return buttonbox
 
-    def saveSize(self):
+    def _saveSize(self, config):
         name = self._window_name
         is_maximized = int(self.windowState()) & QtCore.Qt.WindowMaximized != 0
         if is_maximized:
@@ -275,15 +310,18 @@ class _QBzrWindowBase(object):
             size = geom.width(), geom.height()
         else:
             size = self.width(), self.height()
-        config = QBzrGlobalConfig()
-        config.set_user_option(name + "_window_size", "%dx%d" % size)
-        config.set_user_option(name + "_window_maximized", is_maximized)
-        return config
+        config.set_option(name + "_window_size", "%dx%d" % size)
+        config.set_option(name + "_window_maximized", is_maximized)
+    
+    def saveSize(self):
+        config = get_qbzr_config()
+        self._saveSize(config)
+        config.save()
 
     def restoreSize(self, name, defaultSize):
         self._window_name = name
-        config = QBzrGlobalConfig()
-        size = config.get_user_option(name + "_window_size")
+        config = get_qbzr_config()
+        size = config.get_option(name + "_window_size")
         if size:
             size = size.split("x")
             if len(size) == 2:
@@ -301,21 +339,20 @@ class _QBzrWindowBase(object):
             self.resize(size.expandedTo(self.minimumSizeHint()))
         self._restore_size = size
 
-        is_maximized = config.get_user_option(name + "_window_maximized")
+        is_maximized = config.get_option(name + "_window_maximized")
         if is_maximized in ("True", "1"):
             self.setWindowState(QtCore.Qt.WindowMaximized)
         return config
 
-    def saveSplitterSizes(self):
+    def _saveSplitterSizes(self, config, splitter):
         name = self._window_name
-        config = QBzrGlobalConfig()
-        sizes = ':'.join(map(str, self.splitter.sizes()))
-        config.set_user_option(name + "_splitter_sizes", sizes)
+        sizes = ':'.join(map(str, splitter.sizes()))
+        config.set_option(name + "_splitter_sizes", sizes)
 
     def restoreSplitterSizes(self, default_sizes=None):
         name = self._window_name
-        config = QBzrGlobalConfig()
-        sizes = config.get_user_option(name + "_splitter_sizes")
+        config = get_qbzr_config()
+        sizes = config.get_option(name + "_splitter_sizes")
         n = len(self.splitter.sizes())
         if sizes:
             sizes = map(int, sizes.split(':'))
@@ -370,6 +407,10 @@ class QBzrWindow(QtGui.QMainWindow, _QBzrWindowBase):
         self.windows = []
         self.closing = False
 
+    def show(self):
+        QtGui.QMainWindow.show(self)
+        self.raise_()	# Make sure it displays in the foreground
+
 
 class QBzrDialog(QtGui.QDialog, _QBzrWindowBase):
 
@@ -398,29 +439,23 @@ class QBzrDialog(QtGui.QDialog, _QBzrWindowBase):
         self.saveSize()
         QtGui.QDialog.reject(self)
 
+    def show(self):
+        QtGui.QMainWindow.show(self)
+        self.raise_()	# Make sure it displays in the foreground
+
 throber_movie = None
 
 class ThrobberWidget(QtGui.QWidget):
-    """A window that displays a simple throbber over its parent."""
+    """A widget that indicates activity."""
 
     def __init__(self, parent, timeout=500):
         QtGui.QWidget.__init__(self, parent)
-        self.create_ui()
-        self.num_show = 0
-        
-        # create a timer that displays our window after the timeout.
-        #QtCore.QTimer.singleShot(timeout, self.show)
-
-    def create_ui(self):
-        # a couple of widgets
-        layout = QtGui.QHBoxLayout(self)
-        layout.setContentsMargins(0, 0, 0, 0)
-        
-        self.spinner = QtGui.QLabel("", self)    
         global throber_movie
         if not throber_movie:
             throber_movie = QtGui.QMovie(":/16x16/process-working.gif")
             throber_movie.start()
+        
+        self.spinner = QtGui.QLabel("", self)    
         self.spinner.setMovie(throber_movie)
         
         self.message = QtGui.QLabel(gettext("Loading..."), self)
@@ -429,11 +464,29 @@ class ThrobberWidget(QtGui.QWidget):
         #self.progress.hide()
         #self.progress.setMaximum(sys.maxint)
         self.transport = QtGui.QLabel("", self)
+        for widget in (self.spinner,
+                       self.message,
+                       #self.progress,
+                       self.transport):
+            widget.hide()
+        
+        self.widgets = []
+        self.set_layout()
+        self.num_show = 0
+
+    def set_layout(self):
+        layout = QtGui.QHBoxLayout(self)
+        layout.setContentsMargins(0, 0, 0, 0)
         
         layout.addWidget(self.spinner)
         #layout.addWidget(self.progress)
         layout.addWidget(self.message, 1)
         layout.addWidget(self.transport)
+        
+        self.widgets.append(self.spinner)
+        #self.widgets.append(self.progress)
+        self.widgets.append(self.message)
+        self.widgets.append(self.transport)
 
     def hide(self):
         #if self.is_shown:
@@ -442,12 +495,35 @@ class ThrobberWidget(QtGui.QWidget):
         if self.num_show <= 0:
             self.num_show = 0
             QtGui.QWidget.hide(self)
-
+            for widget in self.widgets:
+                widget.hide()
+    
     def show(self):
         #QtGui.QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
         # and show ourselves.
-        QtGui.QWidget.show(self)
         self.num_show += 1
+        QtGui.QWidget.show(self)
+        for widget in self.widgets:
+            widget.show()
+    
+
+class ToolBarThrobberWidget(ThrobberWidget):
+    """A widget that indicates activity. Smaller than ThrobberWidget, designed
+    for use on a toolbar."""
+
+    def set_layout(self):
+        layout = QtGui.QHBoxLayout(self)
+        layout.setContentsMargins(0, 0, 0, 0)
+        
+        layout.addWidget(self.transport)
+        layout.addWidget(self.spinner)
+        #layout.addWidget(self.progress)
+        #layout.addWidget(self.message, 1)
+        
+        self.widgets.append(self.spinner)
+        #self.widgets.append(self.progress)
+        #self.widgets.append(self.message)
+        self.widgets.append(self.transport)
 
 
 # Helpers for directory pickers.
@@ -484,22 +560,6 @@ def hookup_directory_picker(dialog, chooser, target, chooser_type):
     dialog.connect(chooser, QtCore.SIGNAL("clicked()"), click_handler)
 
 
-_global_config = None
-
-def get_global_config():
-    global _global_config
-    if _global_config is None:
-        _global_config = GlobalConfig()
-    return _global_config
-
-
-def get_branch_config(branch):
-    if branch is not None:
-        return branch.get_config()
-    else:
-        return get_global_config()
-
-
 def open_browser(url):
     try:
         import webbrowser
@@ -640,17 +700,6 @@ class FilterOptions(object):
             return self.renamed or self.modified
         raise ValueError('unknown status: %r' % status)
 
-def split_tokens_at_lines(tokens):
-    currentLine = []
-    for ttype, value in tokens:
-        vsplit = value.splitlines(True)
-        for v in vsplit:
-            currentLine.append((ttype, v))
-            if v[-1:] in ('\n','\r'):
-                yield currentLine
-                currentLine = []
-    yield currentLine
-
 # Some helpers for combo-boxes.  Combos for different purposes (eg, push
 # vs pull) have quite different requirements for the combo:
 # * When pulling from a branch, if the branch is not related to the existing
@@ -700,9 +749,9 @@ def fill_combo_with(combo, default, *iterables):
 def iter_saved_pull_locations():
     """ Iterate the 'pull' locations we have previously saved for the user.
     """
-    config = QBzrConfig()
+    config = get_qbzr_config()
     try:
-        sect = config.getSection('Pull Locations')
+        sect = config.get_section('Pull Locations')
     except KeyError:
         return []
     items = sorted(sect.items())
@@ -733,7 +782,7 @@ def save_pull_location(branch, location):
     max_items = 20
     existing = existing[:max_items]
 
-    config = QBzrConfig()
+    config = get_qbzr_config()
     # and save it to the ini
     section = {}
     for i, save_location in enumerate(existing):
@@ -742,68 +791,10 @@ def save_pull_location(branch, location):
         # edits it)
         key = "%04d" % i
         section[key] = save_location
-    config.setSection('Pull Locations', section)
+    config.set_section('Pull Locations', section)
     config.save()
 
 
-have_pygments = True
-try:
-    from pygments.styles import get_style_by_name
-except ImportError:
-    have_pygments = False
-
-if have_pygments:
-    style = get_style_by_name("default")
-
-def format_for_ttype(ttype, format):
-    if have_pygments and ttype:
-        font = format.font()
-        
-        # If there is no style, use the parent type's style.
-        # It fixes bug 347333 - GaryvdM
-        while not style.styles_token(ttype) and ttype.parent:
-            ttype = ttype.parent
-        
-        tstyle = style.style_for_token(ttype)
-        if tstyle['color']:
-            if isinstance(format, QtGui.QPainter):
-                format.setPen (QtGui.QColor("#"+tstyle['color']))
-            else:
-                format.setForeground (QtGui.QColor("#"+tstyle['color']))
-        if tstyle['bold']: font.setWeight(QtGui.QFont.Bold)
-        if tstyle['italic']: font.setItalic (True)
-        # Can't get this not to affect line height.
-        #if tstyle['underline']: format.setFontUnderline(True)
-        if tstyle['bgcolor']: format.setBackground (QtGui.QColor("#"+tstyle['bgcolor']))
-        # No way to set this for a QTextCharFormat
-        #if tstyle['border']: format.
-    return format
-
-class CachedTTypeFormater(object):
-    def __init__(self, base_format):
-        self.base_format = base_format
-        self._cache = {}
-    
-    def format(self, ttype):
-        if not have_pygments or not ttype:
-            return self.base_format
-        if ttype in self._cache:
-            format = self._cache[ttype]
-        else:
-            format = QtGui.QTextCharFormat(self.base_format)
-            self._cache[ttype] = format
-            
-            # If there is no style, use the parent type's style.
-            # It fixes bug 347333 - GaryvdM
-            while not style.styles_token(ttype) and ttype.parent:
-                ttype = ttype.parent
-                self._cache[ttype] = format
-            
-            format_for_ttype(ttype, format)
-        
-        return format
-
-
 def url_for_display(url):
     """Return human-readable URL or local path for file:/// URLs.
     Wrapper around bzrlib.urlutils.unescape_for_display
@@ -870,8 +861,8 @@ loading_queue = None
 def runs_in_loading_queue(f):
     """Methods decorated with this will not run at the same time, but will be
     queued. Methods decorated with this will not be able to return results,
-    but should rather update the ui themselfs. Methods decorated with this
-    should detect, and stop if their execution is no longer requires.
+    but should rather update the ui themselves. Methods decorated with this
+    should detect, and stop if their execution is no longer required.
     
     """
     
@@ -884,13 +875,17 @@ def run_in_loading_queue(cur_f, *cur_args, **cur_kargs):
     global loading_queue
     if loading_queue is None:
         loading_queue = []
-        loading_queue.append((cur_f, cur_args, cur_kargs))
-        
-        while len(loading_queue):
-            f, args, kargs = loading_queue.pop(0)
-            f(*args, **kargs)
-        
-        loading_queue = None
+        try:
+            loading_queue.append((cur_f, cur_args, cur_kargs))
+            
+            while len(loading_queue):
+                try:
+                    f, args, kargs = loading_queue.pop(0)
+                    f(*args, **kargs)
+                except:
+                    trace.report_exception()
+        finally:
+            loading_queue = None
     else:
         loading_queue.append((cur_f, cur_args, cur_kargs))
 
@@ -984,7 +979,8 @@ def launchpad_project_from_url(url):
     if (host.find('bazaar.launchpad.net') >= 0 or
         host.find('bazaar.launchpad.dev') >= 0):
         parts = path.strip('/').split('/')
-        if len(parts) == 3 and parts[0].startswith('~'):
+        if len(parts) == 3 and (parts[0].startswith('~')
+                                or (parts[0] in ('%2Bbranch', '+branch'))):
             return parts[1]
     return None
 
@@ -1004,3 +1000,161 @@ if MS_WINDOWS:
     shlex_split_unicode = _shlex_split_unicode_windows
 else:
     shlex_split_unicode = _shlex_split_unicode_linux
+
+def get_icon(name):
+    # TODO: Load multiple sizes
+    # TODO: Try load from system theme
+    return QtGui.QIcon(":/22x22/%s.png" % name)
+
+
+class FindToolbar(QtGui.QToolBar):
+    
+    def __init__(self, window, text_edit, show_action):
+        QtGui.QToolBar.__init__(self, gettext("Find"), window)
+        self.text_edit = text_edit
+        if 0: self.text_edit = QtGui.QTextEdit()
+        self.show_action = show_action
+        
+        self.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
+        self.setMovable (False)
+        
+        find_label = QtGui.QLabel(gettext("Find: "), self)
+        self.addWidget(find_label)
+        
+        self.find_text = QtGui.QLineEdit(self)
+        self.addWidget(self.find_text)
+        find_label.setBuddy(self.find_text)
+        
+        prev = self.addAction(get_icon("go-previous"), gettext("Previous"))
+        prev.setShortcut(QtGui.QKeySequence.FindPrevious)
+        
+        next = self.addAction(get_icon("go-next"), gettext("Next"))
+        next.setShortcut(QtGui.QKeySequence.FindNext)
+        
+        self.case_sensitive = QtGui.QCheckBox(gettext("Case sensitive"), self)
+        self.addWidget(self.case_sensitive)
+        self.whole_words = QtGui.QCheckBox(gettext("Whole words"), self)
+        self.addWidget(self.whole_words)
+        
+        close_find = QtGui.QAction(self)
+        close_find.setIcon(self.style().standardIcon(
+                                        QtGui.QStyle.SP_DialogCloseButton))
+        self.addAction(close_find)
+        close_find.setShortcut((QtCore.Qt.Key_Escape))
+        close_find.setShortcutContext(QtCore.Qt.WidgetWithChildrenShortcut)
+        close_find.setStatusTip(gettext("Close find"))
+        self.connect(self.show_action,
+                     QtCore.SIGNAL("toggled (bool)"),
+                     self.show_action_toggle)
+        self.connect(close_find,
+                     QtCore.SIGNAL("triggered(bool)"),
+                     self.close_triggered)
+        self.connect(self.find_text,
+                     QtCore.SIGNAL("textChanged(QString)"),
+                     self.find_text_changed)
+        self.connect(next,
+                     QtCore.SIGNAL("triggered(bool)"),
+                     self.find_next)
+        self.connect(prev,
+                     QtCore.SIGNAL("triggered(bool)"),
+                     self.find_prev)
+        self.connect(self.case_sensitive,
+                     QtCore.SIGNAL("stateChanged(int)"),
+                     self.find_text_changed)
+        self.connect(self.whole_words,
+                     QtCore.SIGNAL("stateChanged(int)"),
+                     self.find_text_changed)
+        self.connect(self.find_text,
+                     QtCore.SIGNAL("returnPressed()"),
+                     self.find_next)        
+        
+    def show_action_toggle(self, state):
+        self.setVisible(state)
+        if state:
+            self.find_text.setFocus()
+        else:
+            self.find_text.setText('')
+    
+    def close_triggered(self, state):
+        self.show_action.setChecked(False)
+    
+    def find_text_changed(self, text):
+        self.find_avoid_moving()
+    
+    def find_get_flags(self):
+        flags = QtGui.QTextDocument.FindFlags()
+        if self.case_sensitive.isChecked():
+            flags = flags | QtGui.QTextDocument.FindCaseSensitively
+        if self.whole_words.isChecked():
+            flags = flags | QtGui.QTextDocument.FindWholeWords
+        return flags
+    
+    def find_avoid_moving(self):
+        self.find(self.text_edit.textCursor().selectionStart(), 0,
+                  self.find_get_flags())
+    
+    def find_next(self, state):
+        self.find(self.text_edit.textCursor().selectionEnd(), 0,
+                  self.find_get_flags())
+    
+    def find_prev(self, state):
+        self.find(self.text_edit.textCursor().selectionStart(),
+                  self.text_edit.document().characterCount(),
+                  self.find_get_flags() | QtGui.QTextDocument.FindBackward)
+    
+    def find(self, from_pos, restart_pos, flags):
+        doc = self.text_edit.document()
+        text = self.find_text.text()
+        cursor = doc.find(text, from_pos, flags)
+        if cursor.isNull():
+            # try again from the restart pos
+            cursor = doc.find(text, restart_pos, flags)
+        if cursor.isNull():
+            cursor = self.text_edit.textCursor()
+            cursor.setPosition(cursor.selectionStart())
+            self.text_edit.setTextCursor(cursor)
+            # Maybe make find_text background red like Firefox?
+        else:
+            self.text_edit.setTextCursor(cursor)
+
+
+class InfoWidget(QtGui.QFrame):
+    def __init__(self, parent=None):
+        QtGui.QFrame.__init__(self, parent)
+        self.setFrameShape(QtGui.QFrame.StyledPanel)
+        
+        self.setAutoFillBackground(True)
+        self.setBackgroundRole(QtGui.QPalette.ToolTipBase) 
+        self.setForegroundRole(QtGui.QPalette.ToolTipText)
+
+
+# Hackish test for monospace. Run bzr qcat lib/util.py to check.
+#888888888888888888888888888888888888888888888888888888888888888888888888888888
+#                                                                             8
+
+monospace_font = None
+def get_monospace_font():
+    global monospace_font
+    if monospace_font is None:
+        monospace_font = _get_monospace_font()
+    return monospace_font
+
+def _get_monospace_font():
+    # TODO: Get font from system settings for Gnome, KDE, Mac.
+    # (no windows option as far as I am aware)
+    # Maybe have our own config setting.
+    
+    # Get the defaul font size
+    size = QtGui.QApplication.font().pointSize()
+    
+    for font_family in ("Monospace", "Courier New"):
+        font = QtGui.QFont(font_family, size)
+        # check that this is really a monospace font
+        if QtGui.QFontInfo(font).fixedPitch():
+            return font
+    
+    # try use style hints to find font.
+    font = QtGui.QFont("", size)
+    font.setFixedPitch(True)
+    return font
+
